/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <iostream>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <interface/XPathNSResolverImpl.hpp>
#include <interface/XPathResultImpl.hpp>
#include <interface/XPathEvaluatorImpl.hpp>
#include "DOMSerialise.hpp"
#include <DOMutils/XStr.hpp>

using namespace std;

int main()
{

	// Initialize the XML4C2 system
	try {
		XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
	}

	catch(const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
		cerr << "Error during Xerces-c Initialization.\n"
		     << "  Exception message:"
		     << XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage()) << endl;
		return 1;
	}


	XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser *xmlParser = new XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser();

	xmlParser->setDoNamespaces(true);
	xmlParser->parse("data.xml");

	const XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *testDoc = xmlParser->getDocument();

	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *tmpNode = testDoc->getDocumentElement();

	try {

		XPathEvaluator *evaluator = new XPathEvaluatorImpl();
		XPathResult *tmpResult;

		XPathNSResolver *docNSResolver = evaluator->createNSResolver(testDoc->getDocumentElement());

		//Find first element from <data> children
		XPathExpression *parsedExpression = evaluator->createExpression(X("/child::*[1]/*"), docNSResolver);

		tmpResult = parsedExpression->evaluate(tmpNode, XPathResult::FIRST_ORDERED_NODE_TYPE, tmpResult);

		XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *elemNode = tmpResult->getSingleNodeValue();

		cout << "Node returned was: " << elemNode << endl;
	}
	catch(const XPathException &e) {
		cout << "XPathException: " << e.getString() << endl;
		return 1;
	}
	catch(const XERCES_CPP_NAMESPACE_QUALIFIER DOMException &e) {
		cout << "XERCES_CPP_NAMESPACE_QUALIFIER DOMException " << e.code << " occured: " << e.msg << endl;
		return 1;
	}

	return 0;
}//main

