/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */


#include <iostream>

#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <interface/XPathNSResolverImpl.hpp>
#include <interface/XPathResultImpl.hpp>
#include <interface/XPathEvaluatorImpl.hpp>
#include <DOMutils/XStr.hpp>
#include "DOMSerialise.hpp"

using namespace std;

int main()
{

	// Initialize the XML4C2 system

	try {
		XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
	}

	catch(const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
		cerr << "Error during Xerces-c Initialization.\n"
		     << "  Exception message:"
		     << XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage()) << endl;
		return 1;
	}

  XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser *xmlParser = new XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser();

	xmlParser->setDoNamespaces(true);
	xmlParser->parse("data.xml");

	const XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *testDoc = xmlParser->getDocument();

	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *tmpNode = testDoc->getDocumentElement();

	try {
    XPathEvaluator *evaluator = new XPathEvaluatorImpl();
		//Find <data> children
		XPathResult *snapshotResult, *iteratorResult;

		XPathNSResolver *docNSResolver = evaluator->createNSResolver(tmpNode);

		XPathExpression *parsedExpression = evaluator->createExpression(X("/child::*[1]/*"), docNSResolver);

		snapshotResult = parsedExpression->evaluate(tmpNode, XPathResult::ORDERED_NODE_SNAPSHOT_TYPE, snapshotResult);
		iteratorResult = parsedExpression->evaluate(tmpNode, XPathResult::ORDERED_NODE_ITERATOR_TYPE, iteratorResult);

		//Now, let's change the tree -- remove the nodes that are in the sets
		while(tmpNode->hasChildNodes() == true) {
			XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *childNode = tmpNode->getFirstChild();
			(void)tmpNode->removeChild(childNode);
		}//while

		//Go through the snapshot, the nodes should still be there
		int totalCount = snapshotResult->getSnapshotLength();
		cout << "Total nodes in set is " << totalCount << endl;

		for(int cnt=0; cnt<totalCount; cnt++) {
			XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *nodeA = snapshotResult->snapshotItem(cnt);

			cout << "Node " << cnt << ": " << nodeA << endl;
		}//for

		cout << endl << endl;

		//Verify the iterator fails
		XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *curNode;
		int tmpCnt = 0;
		while((curNode = iteratorResult->iterateNext())!=NULL) {
			cout << "Node " << tmpCnt << ": " << curNode << endl;
			tmpCnt++;
		}//while
	}
	catch(const XPathException &e) {
		cout << "XPathException: " << e.getString() << endl;
		return 1;
	}
	catch(const XERCES_CPP_NAMESPACE_QUALIFIER DOMException &e) {
		cout << "DOM_DOMException " << e.code << " occured: " << e.msg << endl;
		return 1;
	}

	return 0;
}//main

